@@ -2,8 +2,8 @@ package Tk::DBI::Tree;
 #------------------------------------------------
 # automagically updated versioning variables -- CVS modifies these!
 #------------------------------------------------
-our $Revision           = '$Revision: 1.7 $';
-our $CheckinDate        = '$Date: 2003/06/16 12:58:01 $';
+our $Revision           = '$Revision: 1.11 $';
+our $CheckinDate        = '$Date: 2003/08/13 12:58:13 $';
 our $CheckinUser        = '$Author: xpix $';
 # we need to clean these up right here
 $Revision               =~ s/^\$\S+:\s*(.*?)\s*\$$/$1/sx;
@@ -19,7 +19,7 @@ use Tk::Tree;
 use Tk::Compound;
 use Tk::ItemStyle;
 use Tk::ResizeButton;
-
+use Term::ANSIColor;
 use base qw/Tk::Derived Tk::Frame/;
 
 use strict;
@@ -46,6 +46,7 @@ sub Populate {
 	$obj->{idx}		= delete $args->{'-idx'}	|| return error("No IndexColumn!");
 	$obj->{fields}		= delete $args->{'-fields'} 	|| return error("No Fields!");
 	$obj->{textcolumn}	= delete $args->{'-textcolumn'} || return error("No Textcolumn!");
+	$obj->{joiner}		= delete $args->{'-joiner'};
  	$obj->{start_id}	= delete $args->{'-start_id'} 	|| 1;
  	$obj->{command}		= delete $args->{'-command'};
 	$obj->{parent_id}	= delete $args->{'-parent_id'} 	|| return error("No Parent_id!");
@@ -53,6 +54,10 @@ sub Populate {
 	$obj->{maxchars}	= delete $args->{'-maxchars'};
 	$obj->{colNames}	= delete $args->{'-colNames'};
 	$obj->{entry_create_cb}	= delete $args->{'-entry_create_cb'};
+	$obj->{time_column}	= delete $args->{'-time_column'};
+	$obj->{opencmd}		= delete $args->{'-opencmd'};
+	$obj->{closecmd}	= delete $args->{'-closecmd'};
+
 	my $h_style		= delete $args->{'-highlight'}	|| [-foreground => 'blue'];
 	my $n_style		= delete $args->{'-normal'}	|| [-foreground => 'black'];
 	$obj->{highlight}	= $obj->ItemStyle('imagetext', @{$h_style});
@@ -71,26 +76,41 @@ sub Populate {
 	$specs{infozoom}	= [qw/METHOD infozoom 		InfoZoom/,		undef];
 	$specs{color_all}	= [qw/METHOD color_all 		Color_All/, 		undef];
 	$specs{get_id}		= [qw/METHOD get_id 		Get_Id/, 		undef];
+	$specs{parent_id}	= [qw/METHOD parent_id 		Parent_Id/, 		undef];
+	$specs{see}		= [qw/METHOD see 		See/, 			undef];
+	$specs{childs}		= [qw/METHOD childs 		Childs/, 		undef];
 
 	$specs{neu}		= [qw/METHOD neu 		Neu/, 			undef];
 	$specs{move}		= [qw/METHOD move 		Move/, 			undef];
 	$specs{copy}		= [qw/METHOD copy 		Copy/, 			undef];
 	$specs{dele}		= [qw/METHOD dele 		Dele/, 			undef];
 	$specs{refresh_id}	= [qw/METHOD refresh_id		Refresh_Id/, 		undef];
+	
         $obj->ConfigSpecs(%specs);
 
-
 	$obj->{last_refresh_time} = 1;
 
-	# Bildet den Tree in einem Array ab
-	$obj->{dbtree} = DBIx::Tree->new( 
-		connection => $obj->{dbh}, 
-	        table      => $obj->{table}, 
-	        method     => sub { $obj->make_tree_list(@_) },
-	        columns    => [$obj->{idx}.'+0', $obj->{textcolumn}, $obj->{parent_id}.'+0'],
-	        start_id   => $obj->{start_id},
-        ); 
 
+	# Bildet den Tree in einem Array ab
+	if($obj->{joiner}) {
+		$obj->{dbtree} = DBIx::Tree->new( 
+			connection => $obj->{dbh}, 
+			sql	   => $obj->makeSql,
+		        method     => sub { $obj->make_tree_list(@_) },
+		        columns    => [$obj->{idx}, $obj->{textcolumn}, $obj->{parent_id}],
+		        start_id   => $obj->{start_id},
+			recursive  => 0,
+	        ); 
+	} else {
+		$obj->{dbtree} = DBIx::Tree->new( 
+			connection => $obj->{dbh}, 
+		        table      => $obj->{table}, 
+		        method     => sub { $obj->make_tree_list(@_) },
+		        columns    => [$obj->{idx}.'+0', $obj->{textcolumn}, $obj->{parent_id}.'+0'],
+		        start_id   => $obj->{start_id},
+			recursive  => 0,
+	        ); 
+	}
 	$obj->{tree} = $obj->Scrolled('Tree',
 		-scrollbars 	=> 'ose',
 		-columns	=> scalar @{$obj->{fields}} + 1,
@@ -99,6 +119,11 @@ sub Populate {
 	)->pack(-expand => 1,
 		-fill => 'both');
 
+	$obj->{tree}->configure(-opencmd => sub{ &{$obj->{opencmd}}(@_); $obj->{tree}->OpenCmd(@_)})
+		if(defined $obj->{opencmd} and ref $obj->{opencmd} eq 'CODE');
+	$obj->{tree}->configure(-closecmd => sub{ &{$obj->{closecmd}}(@_); $obj->{tree}->CloseCmd(@_)})
+		if(defined $obj->{closecmd} and ref $obj->{closecmd} eq 'CODE');
+
 	$obj->Advertise("tree" => $obj->{tree});
 
 } # end Populate
@@ -109,11 +134,11 @@ sub Populate {
 sub refresh_id {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
-	my $path 	= shift || return error('No Id');
-	my $to_parent_id 	= shift || return error('No To Id');
-	my $data = shift || return error('No Data');
+	my $path = shift || return error('No Id');
+	my $data = shift || $obj->info('data', $path);
 
-	my ($parent_path, $id) = ($1, $2) if($path =~/(.+)\:(\d+)/);
+	my ($parent_path, $id) = ($1, $2) 
+		if($path =~/(.+)\:(\d+)/);
 	$obj->dele($path);
 	$obj->neu($id, $parent_path, $data);
 }
@@ -148,6 +173,9 @@ sub neu {
 		);
 	}
 	push(@{$obj->{ListOfAllEntries}}, $new_path);
+	$obj->{tree}->setmode($to_parent, 'close')
+		if($obj->{tree}->getmode($to_parent) eq 'none');
+	$obj->{Paths}->{$id} = $new_path;
 	return $new_path;
 }
 
@@ -159,12 +187,18 @@ sub move {
 	my $to_parent 	= shift || return error('No To Id');
 	my $data 	= shift;
 
-	my $to_path = $obj->{Paths}->{$to_parent};
-	my $id = (split( /:/, $from_entry ))[-1];
+	$obj->debug('Move - From Entry: %s to Parent: %s', $from_entry, $to_parent);
+
+	my $to_path = $obj->{Paths}->{int $to_parent} || $obj->{Paths}->{$to_parent};
+	my $id = $obj->id($from_entry);
 
-	my $nid = $obj->neu($id, $to_path, $data);
 	my $did = $obj->dele($from_entry);
-	return $id;
+	my $nid = $obj->neu($id, $to_path, $data);
+
+	$obj->{ListOfAllEntries} = $obj->rm_from_array($obj->{ListOfAllEntries}, $from_entry);
+	push(@{$obj->{ListOfAllEntries}}, $nid);
+
+	return $nid;
 }
 
 # ------------------------------------------
@@ -175,7 +209,7 @@ sub copy {
 	my $to_parent 	= shift || return error('No To Id');
 	my $data 	= shift;
 
-	my $id = (split( /:/, $from_entry ))[-1];
+	my $id = $obj->id($from_entry);
 	my $to_entry = sprintf('%s:%d', $obj->{Paths}->{$to_parent}, $id);
 
 	my $hl = $obj->{tree};
@@ -196,8 +230,9 @@ sub copy {
  		} 
  		$hl->itemCreate($to_entry, $col, @item_args);
     	}
-	$obj->refresh_id($to_entry, $to_parent, $data);
+	$obj->refresh_id($to_entry, $data);
 	push(@{$obj->{ListOfAllEntries}}, $to_entry);
+	$obj->{Paths}->{$id} = $to_entry;
 	return $to_entry;
 }
 
@@ -207,7 +242,15 @@ sub dele {
 	my $obj = shift || return error('No Object');
 	my $id = shift || return error('No Id');
 
+	my $parent = $obj->info('parent', $id)
+		|| return;
 	$obj->{tree}->deleteEntry($id);
+	$obj->{ListOfAllEntries} = $obj->rm_from_array($obj->{ListOfAllEntries}, $id);
+	$obj->{tree}->setmode($parent => 'none')
+		unless( $obj->info('children', $parent) );
+
+	delete $obj->{Paths}->{ $obj->id($id) };
+
 	return $id;
 }
 
@@ -215,10 +258,8 @@ sub dele {
 sub refresh {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
-	my $redraw = shift;
-	
-	return unless($obj->Table_is_Change($obj->{last_refresh_time}));
-
+	my $redraw = shift || $obj->Table_is_Change($obj->{last_refresh_time}, $obj->{table}) || return;
+		
 	unless(defined $obj->{tree_buttons}) {
 		my $c = -1;
 		foreach my $name ($obj->{textcolumn}, @{$obj->{fields}}) {
@@ -252,8 +293,6 @@ sub refresh {
 	$obj->{tree}->configure(-command => $obj->{command})
 		if(defined $obj->{command} and ref $obj->{command} eq 'CODE');
 
-	
-
 	$obj->remember();
 	@{$obj->{ListOfAllEntries}} = ();
 	$obj->{Paths} = {};
@@ -266,6 +305,8 @@ sub refresh {
 		$obj->{zoom} = 0;
 		$obj->zoom();
 	}
+	
+	$obj->{last_refresh_time} = time;
 }
 
 # ------------------------------------------
@@ -277,17 +318,13 @@ sub select_entrys {
 	$obj->zoom if($obj->infozoom);	
 
 	unless(grep(/\:/, @{$obj->{FoundEntrys}})){
-		my @FoundEntrys;
-		foreach my $id (sort @{$obj->{FoundEntrys}} ) {
-			$id = int($id);
-			foreach my $entry (sort @{$obj->listEntrys}) {
-				if($entry =~ /\:${id}$/) {
-					push(@FoundEntrys, $entry);
-					last;
-				}
-			}
+		my @found;
+		foreach my $id (@{$obj->{FoundEntrys}} ) {
+			 my $entry = ($obj->{Paths}->{$id} || $obj->{Paths}->{ int $id });
+			 push(@found, $entry)
+			 	if(defined $entry);
 		}
-		$obj->{FoundEntrys} = \@FoundEntrys;	
+		$obj->{FoundEntrys} = \@found;
 	}		
 
 	foreach (@{$obj->{FoundEntrys}}) { 
@@ -295,12 +332,49 @@ sub select_entrys {
 		$obj->to_parent_open($_);
 		$obj->color_row($_, $obj->{highlight});
 	}
-	my $entry = $obj->{FoundEntrys}->[0];
+	my $entry = $obj->{FoundEntrys}->[0] || return;
+
+	$obj->{tree}->open(1);
 	$obj->{tree}->anchorSet($entry);
 	$obj->{tree}->selectionSet($entry);
 	$obj->{tree}->see($entry);
 }
 
+# ------------------------------------------
+sub childs {
+# ------------------------------------------
+	my $obj = shift || return error('No Object');
+	my $id = shift || return error('No Id');
+
+	return grep(/^$id/, @{$obj->listEntrys()});
+}
+
+# ------------------------------------------
+sub see {
+# ------------------------------------------
+	my $obj = shift || return error('No Object');
+	my $ids = shift || return error('No Id in see()!');
+	my @ret;
+
+	$obj->{tree}->selectionClear();
+	foreach my $id ( split(/[^0-9A-Za-z]/, $ids) ) {
+		my $entry = $obj->{Paths}->{$id} || $obj->{Paths}->{int($id)} || next;
+		$obj->debug('%s: = %s', $id, $entry);
+		$obj->to_parent_open($entry);
+		$obj->{tree}->selectionSet($entry);
+		push(@ret, $entry);
+	}
+
+	my $last = $ret[0] or return error('No Id to see!');
+	$obj->{tree}->open(1);
+	$obj->{tree}->see($last);
+	$obj->{tree}->anchorSet($last);
+	$obj->select_entrys( \@ret )
+		if(scalar @ret > 1);	
+	return @ret;
+}
+
+
 # ------------------------------------------
 sub color_row {
 # ------------------------------------------
@@ -308,6 +382,8 @@ sub color_row {
 	my $id = shift || return error('No Id');
 	my $color = shift || $obj->{normal};
 
+	return unless($obj->info('exists', $id));
+
 	my $i = 0;
 	foreach ($obj->{textcolumn}, @{$obj->{fields}}) {
 		$obj->{tree}->itemConfigure($id, $i, -style => $color);
@@ -361,6 +437,7 @@ sub make_tree_list {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
 	my %parms = @_;
+
 	my @parent_ids = @{ $parms{parent_id} };
 		
 	my $treeval = '';
@@ -407,6 +484,7 @@ sub x2col {
 sub infozoom {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
+	$obj->debug('Zoom is %s', ( $obj->{zoom} ? 'on' : 'off' ));
  	return $obj->{zoom};
 }
 
@@ -414,7 +492,7 @@ sub infozoom {
 sub zoom {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
-	return unless(scalar @{$obj->{FoundEntrys}});
+	return unless($obj->{FoundEntrys});
 	$obj->{zoom} = ($obj->{zoom} ? undef : 1);
 	if($obj->{zoom}) {
 		foreach my $entry (sort @{$obj->{ListOfAllEntries}}) {
@@ -442,13 +520,23 @@ sub zoom {
 sub makeSql {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
+	my $sql;
 
-	my $sql = sprintf('select %s, %s, %s, %s from %s %s ORDER BY %s, %s',
-			$obj->{idx}, $obj->{textcolumn},join(',', @{$obj->{fields}}), $obj->{parent_id},
-			$obj->{table}, 
-			(defined $obj->{where} ? $obj->{where} : ''),
-			$obj->{parent_id}, $obj->{idx}
-			);
+	if($obj->{joiner}) {
+		$sql = sprintf('SELECT %s, %s, %s, %s FROM %s LEFT JOIN %s ON %s ORDER BY %s',
+				  $obj->{idx}, $obj->{textcolumn},join(',', @{$obj->{fields}}), $obj->{parent_id},
+				  $obj->{table}->[0], $obj->{table}->[1],
+				  $obj->{joiner},
+				  $obj->{textcolumn}
+				  );	
+	} else {
+		$sql = sprintf('select %s, %s, %s, %s from %s %s ORDER BY %s, %s',
+				$obj->{idx}, $obj->{textcolumn},join(',', @{$obj->{fields}}), $obj->{parent_id},
+				$obj->{table}, 
+				(defined $obj->{where} ? $obj->{where} : ''),
+				$obj->{parent_id}, $obj->{idx}
+				); 
+	}
 	$obj->debug($sql)
 		if($obj->{debug});
 	return $sql;
@@ -459,16 +547,15 @@ sub getFieldTypes {
 # ------------------------------------------
 	my $obj 	= shift or return warn("No object");
 	my $dbh 	= $obj->{dbh};
-	my $table	= $obj->{table};
+	my $table	= ref $obj->{table} ? $obj->{table}->[0] : $obj->{table};
 
-	return $obj->{fieldtypes}
+	return $obj->{$table}->{fieldtypes}
 		if(defined $obj->{$table}->{fieldtypes});
-
 	
-	my $ret = $dbh->selectall_hashref("show fields from $table", 'Field')
+	$obj->{$table}->{fieldtypes} = $dbh->selectall_hashref("show fields from $table", 'Field')
 		or return $obj->debug($dbh->errstr);
 
-	return $ret;
+	return $obj->{$table}->{fieldtypes};
 }
 
 
@@ -476,7 +563,9 @@ sub getFieldTypes {
 sub list {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
-	my $len = $1 if($obj->getFieldTypes->{$obj->{idx}}->{Type} =~ /(\d+)/);
+	my $idx = ( index($obj->{idx}, '.') ? ( split( '\.', $obj->{idx} ) )[-1] : $obj->{idx});
+
+	my $len = $1 if($obj->getFieldTypes->{$idx}->{Type} =~ /(\d+)/);
 
 	if($DBIx::Tree::VERSION < 1) {
 		$obj->{dbtree}->do_query;	
@@ -486,9 +575,9 @@ sub list {
 	}
 	
 	my $sql = $obj->makeSql;
-        my $DATA = $obj->{dbh}->selectall_hashref( $sql, $obj->{idx} ) 
+
+        my $DATA = $obj->{dbh}->selectall_hashref( $sql, $idx) 
         		or return error($obj->{dbh}->errstr);
-	my $row = $DATA->{$obj->{start_id}} || $DATA->{sprintf("%0${len}d", $obj->{start_id})};		
 
 	foreach my $id (sort @{$obj->{ListOfAllEntries}}) { 
 		my $item_id = (split( /:/, $id ))[-1];
@@ -517,8 +606,9 @@ sub list {
 	$obj->{tree}->autosetmode;
 
 	foreach my $entry (@{$obj->{ListOfAllEntries}}) {
-		$obj->{tree}->close($entry)
-			if(defined $obj->{tree}->{status}->{$entry} and $obj->{tree}->{status}->{$entry} eq 'open');
+		if(defined $obj->{tree}->{status}->{$entry} and $obj->{tree}->{status}->{$entry} eq 'open') {
+			$obj->{tree}->close($entry);
+		}
 	}
 
 }
@@ -527,6 +617,7 @@ sub list {
 sub close_all {
 # ------------------------------------------
 	my $obj = shift || return error('No Object');
+	return unless(ref $obj->{ListOfAllEntries} eq 'ARRAY');
 	foreach my $entry (sort @{$obj->{ListOfAllEntries}}) {
 		$obj->{tree}->close($entry);
 	}
@@ -538,8 +629,10 @@ sub to_parent_open{
 	my $obj = shift || return error('No Object');
 	my $entry = shift;
 	my $path = $obj->{start_id};
+	my $id = $obj->id($entry);
 	foreach my $e (split(/\:/, $entry)) {
 		next if($e eq $obj->{start_id});
+		next if($e eq $id);
 		$path .= sprintf(':%d', $e);		
 		$obj->{tree}->open($path);
 	}
@@ -562,6 +655,26 @@ sub parse_text {
 	return $text;
 } 
 
+# ------------------------------------------
+sub parent_id {
+# ------------------------------------------
+	my $obj = shift || return error('No Object');
+	my $path = shift || return error('No Path');
+	my $parent = $obj->info('parent', $path)
+		or return error('Parent not found!');
+	return (split( /:/, $parent ))[-1];
+}
+
+
+# ------------------------------------------
+sub id {
+# ------------------------------------------
+	my $obj = shift || return error('No Object');
+	my $path = shift || return error('No Path');
+
+	return (split( /:/, $path ))[-1];
+}
+
 # ------------------------------------------
 sub listEntrys {
 # ------------------------------------------
@@ -572,9 +685,22 @@ sub listEntrys {
 # ------------------------------------------
 sub info {
 # ------------------------------------------
-	my $obj = shift;
-	my $typ = shift || 'data';
+	my $obj = shift or return error("No object");
+	my $typ = shift or return error("No Type");
 	my $entry = shift;
+
+	if($typ =~ /^(selection|anchor|dragsite|dropsite)$/si) {
+		my @ids = $obj->{tree}->info($typ);
+		return \@ids;
+	}
+
+	if($entry !~ /\:/) {
+		$entry = $obj->{Paths}->{$entry}
+			or return error('Can\'t find <%s> in Paths!', $entry);
+	}
+
+	return error('Can\'t find Id: %s', $entry)
+		if($typ ne 'exists' and ! $obj->{tree}->info('exists', $entry));
 	return $obj->{tree}->info(${typ}, $entry);
 } 
 
@@ -596,7 +722,7 @@ sub Table_is_Change {
 # ------------------------------------------
 	my $obj 	= shift or return error("No object");
 	my $lasttime	= shift || $obj->{last_refresh_time};	# No last time, first request!
-	my $table	= shift || $obj->{table};
+	my $table	= shift || $obj->{table} || $obj->{table}->[0];
 
 	my $dbh 	= $obj->{dbh};
 	my $ret = 0;
@@ -604,23 +730,39 @@ sub Table_is_Change {
 	my $data = $dbh->selectall_hashref(sprintf("SHOW TABLE STATUS LIKE '%s'", $table),'Name')
 		or return $obj->debug($dbh->errstr);
 
-	my $unixtime = $obj->getSqlArray(sprintf("select UNIX_TIMESTAMP('%s')", $data->{$table}->{Update_time}));
+	my $unixtime = $obj->getSqlArray(sprintf("select UNIX_TIMESTAMP('%s')", $data->{$table}->{Update_time}))
+		if(defined $data->{$table}->{Update_time});
 
 	$obj->{last_refresh_time} = time;
 
-	if($unixtime->[0][0] > $lasttime) {
+	if(defined $unixtime->[0][0] and $unixtime->[0][0] > $lasttime) {
 		return 1;
 	}
 }
 
+# ------------------------------------------
+sub rm_from_array {
+# ------------------------------------------
+	my $obj = shift || return error('No Object');
+	my $arr = shift || return error('No Array');
+	my $id  = shift || return error('No Id');
+	
+	my @new_array = grep(!/$id/, @$arr);
+	
+	return \@new_array;
+}
+
+
 
 # ------------------------------------------
 sub debug {
 # ------------------------------------------
 	my $obj = shift;
 	my $msg = shift || return;
-	printf($msg, @_); 
-	print "\n";
+	print color 'green';
+	print 'Tk::Tree: ', sprintf($msg, @_), "\n"
+		if($obj->{debug});
+	print color 'reset';
 } 
 
 # ------------------------------------------
@@ -629,9 +771,11 @@ sub error {
 	my $msg = shift;
         my ($package, $filename, $line, $subroutine, $hasargs,
                 $wantarray, $evaltext, $is_require, $hints, $bitmask) = caller(1);
-	my $error = sprintf("ERROR in %s:%s #%d: %s",
+	my $error = sprintf("ERROR in %s:%s #%d: <%s>\n",
                 $package, $subroutine, $line, sprintf($msg, @_));
-	warn $error;
+	print color 'bold red';
+	print $error;
+	print color 'reset';
 	return undef;
 } 
 
@@ -712,31 +856,6 @@ The id, where the widget will start to create the tree. Default is 1.
 
 Default field column width.
 
-=head2 -command => sub{ ... }
-
-Callback on TreeWidget at browsing.
-
-=head2 -entry_create_cb => sub{ ... }
-
-Callback if an entry created. The routine have 2 parameters:
-
-=over 4
-
-=item entry - a ref to created entry
-
-=item data - a ref hash with row information.
-
-=back
-
-i.e;
-
-  -entry_create_cb => sub{
-	my($w, $path, $row) = @_;
-	if(exists $DOC->{ $row->{id} } and exists $EVENT->{ $row->{id} } ) {
-		$w->entryconfigure($path, -image => $pics{'icon_document_event'});
-	}
-  },
-
 =head2 -highlight => I<[-foreground => 'blue']>
 
 Style for founded Entries.
@@ -747,6 +866,20 @@ Default style for Entries.
 
 =head2 -maxchars => number or {col1 =number}
 
+Maximum number of characters to be displayed within the cells. Global
+validity or set only for named columns.
+I.E.:
+
+  -maxchars => {
+	 descr => 25,
+	 name => 10,
+  },
+  # or ....
+  -maxchars => 25, # global for all fields
+
+
+=head2 -time_column => $name_from_time_column
+
 Maximum number of characters to be displayed within the cells. Global
 validity or set only for named columns.
 I.E.:
@@ -769,6 +902,10 @@ then this will reload the table from database. If you call this without paramete
 look this widget is the table changed (update date) at the last refresh. If this true, then
 load this the complete table and redraw the tree.
 
+=head2 $DBITree->refresh_id( I<entry>, I<data> );
+
+This will refresh (delete -> new) a Tree item.
+
 =head2 $DBITree->copy( I<entry>, I<to_parent_entry>, I<data> );
 
 Copy an entry (entry) to a parent branch (to_parent_entry) with data (data);
@@ -850,6 +987,17 @@ select the row under mouseposition and returnd following parameters.
 
 =back
 
+=head2 $DBITree->childs($item);
+
+Return an array with paths from childs for $item, include $item.
+
+=head2 $DBITree->parent_id($path);
+
+Return the actually id from the parentree (only the integer id, not the hole path)
+
+=head2 $DBITree->see($id);
+
+Jump to id (only database id)
 
 =head2 $DBITree->remember( $hash );
 
@@ -883,6 +1031,44 @@ I.E.:
  	 widths =[165, 24, 546],
   } );
 
+=head1 CALLBACKS
+
+=head2 -command => sub{ ... }
+
+Callback on TreeWidget at browsing.
+
+=head2 -entry_create_cb => sub{ ... }
+
+Callback if an entry created. The routine have 2 parameters:
+
+=over 4
+
+=item entry - a ref to created entry
+
+=item data - a ref hash with row information.
+
+=back
+
+i.e;
+
+  -entry_create_cb => sub{
+	my($w, $path, $row) = @_;
+	if(exists $DOC->{ $row->{id} } and exists $EVENT->{ $row->{id} } ) {
+		$w->entryconfigure($path, -image => $pics{'icon_document_event'});
+	}
+  },
+
+=head2 -opencmd => sub{ ... }
+
+Callback on TreeWidget if this entry activate(from User or Automatic) to open.
+i.E.
+	-opencmd	=> sub{ &display_prj_items( @_ ) },
+
+
+=head2 -closecmd => sub{ ... }
+
+Callback on TreeWidget if this entry activate(from User or Automatic) to close.
+
 
 =head1 ADVERTISED WIDGETS
 
@@ -901,6 +1087,33 @@ This is a (Resize)Button widget.
 =head1 CHANGES
 
   $Log: Tree.pm,v $
+  Revision 1.11  2003/08/13 12:58:13  xpix
+  * colored debug output
+  * new method childs, output an array with complete paths from childs
+  * new methods neu, move, dele to transfering from entrys
+  * new option maxchars
+  ! many, many bug fixes
+
+  Revision 1.8  2003/07/18 16:14:15  xpix
+  ! Fehler im Table_is_Change algo, fix
+  ! Komplett refresh bei TreeStatusaenderung
+  ! to_parent_open macht jetzt wirklich nur die Vaeter und nicht noch den Sohn auf ;-)
+  ! unit. value in Form.pm
+
+  Revision 1.10  2003/07/17 14:59:54  xpix
+  ! many little bugfixes
+
+  Revision 1.3  2003/06/24 16:38:44  xpix
+  * add symbolic to cvs
+  * new loking mechanism in Form.pm
+
+  Revision 1.9  2003/06/23 16:15:22  xpix
+  ! cvs error
+
+  Revision 1.8  2003/06/18 15:31:47  xpix
+  * new methods: copy, move, neu ... dele. This will work only on the Tree Widget (not in table)
+  * change a little bit the docu
+
   Revision 1.7  2003/06/16 12:58:01  xpix
   ! No Error, if the id ot exists in selct_entrys
 
